///////////////////////////////////////////////////////////////////////////////
//
//  Authors:	Jeffrey Hanke (hanke@cs)
//		Jason Dang (jaydang@cs)
//		Chris Baker (crbaker@cs)
//
//  File:	Queue.java	
//
//  Abstract:	Implements the queue ADT with linked lists
//
///////////////////////////////////////////////////////////////////////////////

public class Queue
{

  private Node head;
  private Node rear; 

  Queue() 
  {
    head = null;
    rear = head;
  }

  //
  //  Method:	isEmpty()
  //
  //  Abstract:	Determines if the queue is empty
  //
  public boolean isEmpty()
  {
    return null == head;
  }

  //
  //  Method:	makeEmpty()
  //
  //  Abstract:	Make the queue empty
  //
  public void makeEmpty()
  {
    head = null;
  }


  //
  //  Method:	getFront()
  //
  //  Abstract:	returns the front element of the queue
  //
  public Object getFront()
  {
    if( !isEmpty() )
      return head.element;
    else
      return null; 
  }

  public void addFirst(Object element)
  {
  	if( isEmpty() )
	{
	  head = new Node( element );
	  rear = head;
	}
	else
	{
	  head = new Node( element, head);
	}
  }
  
  //
  //  Method:	enqueue()
  //
  //  Abstract:	Add an element to the queue
  //
  public void enqueue( Object element )
  {
    if( isEmpty() )
    {
      head = new Node( element );
      rear = head;
    }
    else
    {
      rear.next = new Node( element );
      rear = rear.next;
    }
  }

  //
  //  Method:	dequeue()
  //
  //  Abstract:	Removes and returns the first element in the queue
  //
  public Object dequeue()
  {
    Object temp = null;
    if( !isEmpty() )
    {
      temp = head.element;
      head = head.next;
    }
    return temp;
  }

  //
  //  Method:	inQueue()
  //
  //  Abstract:	Determines if an element is already in the queue	
  //
  public boolean inQueue( Object element )
  {
    Node tmp = head;
    while( null != tmp )
    {
      if( tmp.element.equals( element ) )
        return true;
      tmp = tmp.next;
    }
    return false;
  }
}

